/*
 * serial.c
 *
 *  Created on: 29.11.2019
 *      Author: pm
 */

#include <msp430.h>
#include <stdio.h>
#include <strings.h>
#include <stdint.h>
#include <stdbool.h>
#include "fifo.h"


FIFO_T txFifo;
FIFO_T rxFifo;
static uint8_t rxdata[16], txdata[16];

void serialInit(void){
    // Configure UART pins
    P4SEL0 |= BIT2 | BIT3;                    // set UART pin as second function

    // Configure UART
    UCA1CTLW0 |= UCSWRST;
    UCA1CTLW0 |= UCSSEL__SMCLK;

    // Baud Rate calculation for 9600 kBit
    // 8000000/(16*9600) = 52.083
    // Fractional portion = 0.083
    // User's Guide Table 17-4: UCBRSx = 0x49
    // UCBRFx = int ( (52.083-52)*16) = 1
    UCA1BR0 = 52;                             // 8000000/16/9600
    UCA1BR1 = 0x00;
    UCA1MCTLW = 0x4900 | UCOS16 | UCBRF_1;

    UCA1CTLW0 &= ~UCSWRST;                    // Initialize eUSCI
    UCA1IE |= UCRXIE;                         // Enable USCI_A0 RX interrupt

    fifoInit(&rxFifo, rxdata ,sizeof(rxdata)/sizeof(uint8_t));
}



// Send a character via serial interface. If no transmission is running
// the character is directly put into the tx register, otherwise enqueued.
int fputc(int _c, register FILE *_fp)
{
    while(!(UCA1IFG & UCTXIFG));
    UCA1TXBUF = _c;

  return((unsigned char)_c);
}

int fputs(const char *_ptr, register FILE *_fp)
{
  unsigned int i, len;

  len = strlen(_ptr);

  for(i=0 ; i<len ; i++)
  {
      fputc((unsigned char) _ptr[i], _fp);
  }

  return len;
}


#pragma vector=USCI_A1_VECTOR
__interrupt void USCI_A1_ISR(void)
{
  switch(__even_in_range(UCA1IV,4))
  {
  case 0:
      break;                             // Vector 0 - no interrupt

  case 2:                                   // Vector 2 - RXIFG
      fifoPut(&rxFifo, UCA1RXBUF); // store received byte. If buffer is full nothing happens
    break;

  case 4:                             // Vector 4 - TXIFG
     break;

  default: break;
  }
}
